var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { ModelFactory } from './model.factory';
import { Relationship } from './relationship';
import { Models } from './models';
var BaseModel = (function () {
    function BaseModel() {
        this.modelConfig = {
            key: '',
            uri: '',
            hidden: [],
            relationships: {}
        };
    }
    BaseModel.prototype.setConfig = function (config) {
        this.modelConfig = __assign({}, this.getConfig(), config);
    };
    BaseModel.prototype.getConfig = function () {
        return this.modelConfig;
    };
    BaseModel.prototype.getUri = function () {
        return this.modelConfig.uri;
    };
    BaseModel.prototype.getSelfUri = function () {
        return this.modelConfig.uri + '/' + this[this.modelConfig.key];
    };
    BaseModel.prototype.setUri = function (uri) {
        this.modelConfig.uri = uri;
    };
    BaseModel.prototype.getKey = function () {
        return this.modelConfig.key;
    };
    BaseModel.prototype.getRelationship = function (key) {
        return this.modelConfig.relationships[key];
    };
    BaseModel.prototype.deserialize = function (input) {
        var config = this.getConfig();
        var relationships = [];
        for (var prop in input) {
            if (!input.hasOwnProperty(prop)) {
                continue;
            }
            if (config.hidden.indexOf(prop) > -1) {
                continue;
            }
            if (typeof input[prop] === 'object') {
                if (config.relationships.hasOwnProperty(this.snakeToCamel(prop))) {
                    relationships.push({
                        key: this.snakeToCamel(prop),
                        value: (input[prop] ? input[prop].data : null)
                    });
                }
                continue;
            }
            this[this.snakeToCamel(prop)] = input[prop];
        }
        for (var _i = 0, relationships_1 = relationships; _i < relationships_1.length; _i++) {
            var relationship = relationships_1[_i];
            if (relationship.value === null) {
                continue;
            }
            var relationshipConfig = config.relationships[relationship.key];
            var related = Models.get(relationshipConfig.model);
            var relationshipObject = new Relationship(this, related, relationshipConfig.type);
            this[relationship.key] = ModelFactory.makeRelatedFromRelationship(relationshipObject, relationship.value);
        }
        return this;
    };
    BaseModel.prototype.snakeToCamel = function (string) {
        return string.replace(/_\w/g, function (m) {
            return m[1].toUpperCase();
        });
    };
    return BaseModel;
}());
export { BaseModel };
//# sourceMappingURL=base-model.js.map